require 'crowdfund/project'
require 'crowdfund/pledge_pool'

module CrowdFund
  describe Project do
    before do
      $stdout = StringIO.new
      @initial_funding = 150
      @initial_backers = 1
      @funding_goal = 10000
      @name = "hover car"
      @project = Project.new(@name, @funding_goal, @initial_funding, @initial_backers)
    end
  
    it "has a capitalized name" do
      expect(@project.name).to eq(@name.capitalize)
    end
  
    it "has an initial target funding goal" do
      expect(@project.funding_goal).not_to eq(0)
    end
  
    it "computes the total funding outstanding as the target funding amount minus the funding amount" do
      expect(@project.funding_gap).to eq(@project.funding_goal - @initial_funding)
    end
  
    it "increases funds by 25 when funds are added" do
      amount = 25
      expect{ @project.add_funds(amount) }.to change(@project, :funding).by(amount)
    end
  
    it "increases backers by 1 when funds are added" do
      amount = 25
      expect{ @project.add_funds(amount) }.to change(@project, :backers).by(1)
    end

    it "decreases funds by 15 when funds are removed" do
      amount = 15
      expect{ @project.remove_funds(amount) }.to change(@project, :funding).by(-amount)
    end

    it "decreases backers by 1 when funds are removed" do
      amount = 15
      expect{ @project.remove_funds(amount) }.to change(@project, :backers).by(-1)
    end

    it "is not fully funded when funding < funding_goal" do
      expect(@project.funded?).to eq(false)
    end

    it "yields each pledge level and its total value" do
      @project.record_pledge(Pledge.new(:gold, 100))
      @project.record_pledge(Pledge.new(:gold, 100))
      @project.record_pledge(Pledge.new(:silver, 75))
      @project.record_pledge(Pledge.new(:bronze, 50))
      @project.record_pledge(Pledge.new(:bronze, 50))
      @project.record_pledge(Pledge.new(:bronze, 50))
      @project.record_pledge(Pledge.new(:base, 25))
      @project.record_pledge(Pledge.new(:base, 25))

      pledged = []
      @project.each_donated_pledge do |pledge|
        pledged << pledge
      end

      expect(pledged).to eq([
        Pledge.new(:gold, 200),
        Pledge.new(:silver, 75),
        Pledge.new(:bronze, 150),
        Pledge.new(:base, 50)
      ])
    end

    it "can be created from a CSV string" do
      aggregate_failures "from_csv record" do
        project = Project.from_csv(["jon", "200", "0", "0"])
  
        expect(project.name).to eq("Jon")
        expect(project.funding_goal).to eq(200)
        expect(project.funding).to eq(0)
        expect(project.backers).to eq(0)
      end
    end
  
  
    context "created with a default funding amount" do
      before do
        @project = Project.new(@name, @funding_goal)
      end
    
      it "has a default value of 0 for funding amount" do
        expect(@project.funding).to eq(0)
      end
    end

    context "fully funded" do
      before do
        @initial_funding = @funding_goal
        @project = Project.new(@name, @funding_goal, @initial_funding, @intial_backers)
      end
   
      it "is fully funded when funding gap <= 0" do
        expect(@project.funded?).to eq(true)
      end
    end
  end
end
